from typing import Final

API_BASE_URL: Final[str] = "https://api.lanyard.rest/"
"""Base URL for Lanyard API"""

API_DEFAULT_VERSION: Final[int] = 1
"""Default API version"""

__all__ = [
    "API_BASE_URL",
    "API_DEFAULT_VERSION",
]
